@@ -3,14 +3,14 @@
 
 from __future__ import print_function
 
-import logging
 import json
+import logging
 import os
-import re
+
 import requests
-import yaml
 import rich.console
 import rich.table
+import yaml
 
 import nf_core.utils
 
@@ -58,8 +58,8 @@ class WorkflowLicences(object):
                 )
             self.conda_config = pipeline_obj.conda_config
         else:
-            env_url = "https://raw.githubusercontent.com/nf-core/{}/master/environment.yml".format(self.pipeline)
-            log.debug("Fetching environment.yml file: {}".format(env_url))
+            env_url = f"https://raw.githubusercontent.com/nf-core/{self.pipeline}/master/environment.yml"
+            log.debug(f"Fetching environment.yml file: {env_url}")
             response = requests.get(env_url)
             # Check that the pipeline exists
             if response.status_code == 404:
@@ -74,7 +74,7 @@ class WorkflowLicences(object):
         # Check conda dependency list
         deps = self.conda_config.get("dependencies", [])
         deps_data = {}
-        log.info("Fetching licence information for {} tools".format(len(deps)))
+        log.info(f"Fetching licence information for {len(deps)} tools")
         for dep in deps:
             try:
                 if isinstance(dep, str):
@@ -83,10 +83,10 @@ class WorkflowLicences(object):
                 elif isinstance(dep, dict):
                     deps_data[dep] = nf_core.utils.pip_package(dep)
             except ValueError:
-                log.error("Couldn't get licence information for {}".format(dep))
+                log.error(f"Couldn't get licence information for {dep}")
 
         for dep, data in deps_data.items():
-            depname, depver = dep.split("=", 1)
+            _, depver = dep.split("=", 1)
             self.conda_package_licences[dep] = nf_core.utils.parse_anaconda_licence(data, depver)
 
     def print_licences(self):
