@@ -1,7 +1,10 @@
 #!/usr/bin/env python
 
+import logging
 import os
 
+log = logging.getLogger(__name__)
+
 
 def files_exist(self):
     """Checks a given pipeline directory for required files.
@@ -22,6 +25,7 @@ def files_exist(self):
         .gitignore
         .nf-core.yml
         .editorconfig
+        .prettierignore
         .prettierrc.yml
         .github/.dockstore.yml
         .github/CONTRIBUTING.md
@@ -110,12 +114,18 @@ def files_exist(self):
     # NB: Should all be files, not directories
     # List of lists. Passes if any of the files in the sublist are found.
     #: test autodoc
-    short_name = self.nf_config["manifest.name"].strip("\"'").replace("nf-core/", "")
+    try:
+        _, short_name = self.nf_config["manifest.name"].strip("\"'").split("/")
+    except ValueError:
+        log.warning("Expected manifest.name to be in the format '<repo>/<pipeline>'. Will assume it is '<pipeline>'.")
+        short_name = self.nf_config["manifest.name"].strip("\"'").split("/")
+
     files_fail = [
         [".gitattributes"],
         [".gitignore"],
         [".nf-core.yml"],
         [".editorconfig"],
+        [".prettierignore"],
         [".prettierrc.yml"],
         ["CHANGELOG.md"],
         ["CITATIONS.md"],
@@ -198,39 +208,39 @@ def files_exist(self):
         if any([f in ignore_files for f in files]):
             continue
         if any([os.path.isfile(pf(f)) for f in files]):
-            passed.append("File found: {}".format(self._wrap_quotes(files)))
+            passed.append(f"File found: {self._wrap_quotes(files)}")
         else:
-            failed.append("File not found: {}".format(self._wrap_quotes(files)))
+            failed.append(f"File not found: {self._wrap_quotes(files)}")
 
     # Files that cause a warning if they don't exist
     for files in files_warn:
         if any([f in ignore_files for f in files]):
             continue
         if any([os.path.isfile(pf(f)) for f in files]):
-            passed.append("File found: {}".format(self._wrap_quotes(files)))
+            passed.append(f"File found: {self._wrap_quotes(files)}")
         else:
-            warned.append("File not found: {}".format(self._wrap_quotes(files)))
+            warned.append(f"File not found: {self._wrap_quotes(files)}")
 
     # Files that cause an error if they exist
     for file in files_fail_ifexists:
         if file in ignore_files:
             continue
         if os.path.isfile(pf(file)):
-            failed.append("File must be removed: {}".format(self._wrap_quotes(file)))
+            failed.append(f"File must be removed: {self._wrap_quotes(file)}")
         else:
-            passed.append("File not found check: {}".format(self._wrap_quotes(file)))
+            passed.append(f"File not found check: {self._wrap_quotes(file)}")
 
     # Files that cause a warning if they exist
     for file in files_warn_ifexists:
         if file in ignore_files:
             continue
         if os.path.isfile(pf(file)):
-            warned.append("File should be removed: {}".format(self._wrap_quotes(file)))
+            warned.append(f"File should be removed: {self._wrap_quotes(file)}")
         else:
-            passed.append("File not found check: {}".format(self._wrap_quotes(file)))
+            passed.append(f"File not found check: {self._wrap_quotes(file)}")
 
     # Files that are ignoed
     for file in ignore_files:
-        ignored.append("File is ignored: {}".format(self._wrap_quotes(file)))
+        ignored.append(f"File is ignored: {self._wrap_quotes(file)}")
 
     return {"passed": passed, "warned": warned, "failed": failed, "ignored": ignored}
