@@ -5,28 +5,29 @@ or for a single module
 
 
 from __future__ import print_function
+
 import logging
-import questionary
-import os
 import re
+
+import questionary
 import rich
 from rich.console import Console
-from rich.table import Table
 from rich.markdown import Markdown
-import rich
+from rich.table import Table
+
+import nf_core.modules.module_utils
+import nf_core.utils
+from nf_core.utils import plural_s as _s
 from nf_core.utils import rich_force_colors
 
-import nf_core.utils
-import nf_core.modules.module_utils
-from nf_core.modules.nfcore_module import NFCoreModule
 from .modules_command import ModuleCommand
 
 log = logging.getLogger(__name__)
 
 
 class ModuleVersionBumper(ModuleCommand):
-    def __init__(self, pipeline_dir):
-        super().__init__(pipeline_dir)
+    def __init__(self, pipeline_dir, remote_url=None, branch=None, no_pull=False):
+        super().__init__(pipeline_dir, remote_url, branch, no_pull)
 
         self.up_to_date = None
         self.updated = None
@@ -110,7 +111,7 @@ class ModuleVersionBumper(ModuleCommand):
 
         self._print_results()
 
-    def bump_module_version(self, module: NFCoreModule):
+    def bump_module_version(self, module):
         """
         Bump the bioconda and container version of a single NFCoreModule
 
@@ -123,7 +124,7 @@ class ModuleVersionBumper(ModuleCommand):
 
         # If multiple versions - don't update! (can't update mulled containers)
         if not bioconda_packages or len(bioconda_packages) > 1:
-            self.failed.append((f"Ignoring mulled container", module.module_name))
+            self.failed.append(("Ignoring mulled container", module.module_name))
             return False
 
         # Don't update if blocked in blacklist
@@ -131,7 +132,7 @@ class ModuleVersionBumper(ModuleCommand):
         if module.module_name in self.bump_versions_config:
             config_version = self.bump_versions_config[module.module_name]
             if not config_version:
-                self.ignored.append((f"Omitting module due to config.", module.module_name))
+                self.ignored.append(("Omitting module due to config.", module.module_name))
                 return False
 
         # check for correct version and newer versions
@@ -143,7 +144,7 @@ class ModuleVersionBumper(ModuleCommand):
         if not config_version:
             try:
                 response = nf_core.utils.anaconda_package(bp)
-            except (LookupError, ValueError) as e:
+            except (LookupError, ValueError):
                 self.failed.append((f"Conda version not specified correctly: {module.main_nf}", module.module_name))
                 return False
 
@@ -168,9 +169,9 @@ class ModuleVersionBumper(ModuleCommand):
 
             patterns = [
                 (bioconda_packages[0], f"'bioconda::{bioconda_tool_name}={last_ver}'"),
-                (r"quay.io/biocontainers/{}:[^'\"\s]+".format(bioconda_tool_name), docker_img),
+                (rf"quay.io/biocontainers/{bioconda_tool_name}:[^'\"\s]+", docker_img),
                 (
-                    r"https://depot.galaxyproject.org/singularity/{}:[^'\"\s]+".format(bioconda_tool_name),
+                    rf"https://depot.galaxyproject.org/singularity/{bioconda_tool_name}:[^'\"\s]+",
                     singularity_img,
                 ),
             ]
@@ -185,7 +186,7 @@ class ModuleVersionBumper(ModuleCommand):
                 for line in content.splitlines():
 
                     # Match the pattern
-                    matches_pattern = re.findall("^.*{}.*$".format(pattern[0]), line)
+                    matches_pattern = re.findall(rf"^.*{pattern[0]}.*$", line)
                     if matches_pattern:
                         found_match = True
 
@@ -225,26 +226,16 @@ class ModuleVersionBumper(ModuleCommand):
         Extract the bioconda version from a module
         """
         # Check whether file exists and load it
-        bioconda_packages = None
+        bioconda_packages = False
         try:
             with open(module.main_nf, "r") as fh:
-                lines = fh.readlines()
-        except FileNotFoundError as e:
+                for l in fh:
+                    if "bioconda::" in l:
+                        bioconda_packages = [b for b in l.split() if "bioconda::" in b]
+        except FileNotFoundError:
             log.error(f"Could not read `main.nf` of {module.module_name} module.")
-            return False
 
-        for l in lines:
-            if re.search("bioconda::", l):
-                bioconda_packages = [b for b in l.split() if "bioconda::" in b]
-            if re.search("org/singularity", l):
-                singularity_tag = l.split("/")[-1].replace('"', "").replace("'", "").split("--")[-1].strip()
-            if re.search("biocontainers", l):
-                docker_tag = l.split("/")[-1].replace('"', "").replace("'", "").split("--")[-1].strip()
-
-        if bioconda_packages:
-            return bioconda_packages
-        else:
-            return False
+        return bioconda_packages
 
     def _print_results(self):
         """
@@ -264,11 +255,6 @@ class ModuleVersionBumper(ModuleCommand):
             except:
                 pass
 
-        def _s(some_list):
-            if len(some_list) > 1:
-                return "s"
-            return ""
-
         def format_result(module_updates, table):
             """
             Create rows for module updates
@@ -295,9 +281,7 @@ class ModuleVersionBumper(ModuleCommand):
         if len(self.up_to_date) > 0 and self.show_up_to_date:
             console.print(
                 rich.panel.Panel(
-                    r"[!] {} Module{} version{} up to date.".format(
-                        len(self.up_to_date), _s(self.up_to_date), _s(self.up_to_date)
-                    ),
+                    rf"[!] {len(self.up_to_date)} Module{_s(self.up_to_date)} version{_s(self.up_to_date)} up to date.",
                     style="bold green",
                 )
             )
@@ -310,9 +294,7 @@ class ModuleVersionBumper(ModuleCommand):
         # Table of updated modules
         if len(self.updated) > 0:
             console.print(
-                rich.panel.Panel(
-                    r"[!] {} Module{} updated".format(len(self.updated), _s(self.updated)), style="bold yellow"
-                )
+                rich.panel.Panel(rf"[!] {len(self.updated)} Module{_s(self.updated)} updated", style="bold yellow")
             )
             table = Table(style="yellow", box=rich.box.ROUNDED)
             table.add_column("Module name", width=max_mod_name_len)
@@ -323,9 +305,7 @@ class ModuleVersionBumper(ModuleCommand):
         # Table of modules that couldn't be updated
         if len(self.failed) > 0:
             console.print(
-                rich.panel.Panel(
-                    r"[!] {} Module update{} failed".format(len(self.failed), _s(self.failed)), style="bold red"
-                )
+                rich.panel.Panel(rf"[!] {len(self.failed)} Module update{_s(self.failed)} failed", style="bold red")
             )
             table = Table(style="red", box=rich.box.ROUNDED)
             table.add_column("Module name", width=max_mod_name_len)
@@ -336,9 +316,7 @@ class ModuleVersionBumper(ModuleCommand):
         # Table of modules ignored due to `.nf-core.yml`
         if len(self.ignored) > 0:
             console.print(
-                rich.panel.Panel(
-                    r"[!] {} Module update{} ignored".format(len(self.ignored), _s(self.ignored)), style="grey58"
-                )
+                rich.panel.Panel(rf"[!] {len(self.ignored)} Module update{_s(self.ignored)} ignored", style="grey58")
             )
             table = Table(style="grey58", box=rich.box.ROUNDED)
             table.add_column("Module name", width=max_mod_name_len)
