@@ -8,30 +8,25 @@ nf-core modules lint
 """
 
 from __future__ import print_function
+
 import logging
-from nf_core.modules.modules_command import ModuleCommand
 import operator
 import os
+from pathlib import Path
+
 import questionary
-import re
-import requests
 import rich
-import yaml
-import json
-from rich.table import Table
 from rich.markdown import Markdown
-from rich.panel import Panel
-import rich
-from nf_core.utils import rich_force_colors
-from nf_core.lint.pipeline_todos import pipeline_todos
-import sys
+from rich.table import Table
 
-import nf_core.utils
 import nf_core.modules.module_utils
-
+import nf_core.utils
+from nf_core.lint_utils import console
+from nf_core.modules.modules_command import ModuleCommand
+from nf_core.modules.modules_json import ModulesJson
 from nf_core.modules.modules_repo import ModulesRepo
 from nf_core.modules.nfcore_module import NFCoreModule
-from nf_core.lint_utils import console
+from nf_core.utils import plural_s as _s
 
 log = logging.getLogger(__name__)
 
@@ -63,44 +58,88 @@ class ModuleLint(ModuleCommand):
     from .main_nf import main_nf
     from .meta_yml import meta_yml
     from .module_changes import module_changes
+    from .module_deprecations import module_deprecations
+    from .module_patch import module_patch
     from .module_tests import module_tests
     from .module_todos import module_todos
-    from .module_deprecations import module_deprecations
     from .module_version import module_version
 
-    def __init__(self, dir):
-        self.dir = dir
-        try:
-            self.dir, self.repo_type = nf_core.modules.module_utils.get_repo_type(self.dir)
-        except LookupError as e:
-            raise UserWarning(e)
+    def __init__(
+        self,
+        dir,
+        fail_warned=False,
+        remote_url=None,
+        branch=None,
+        no_pull=False,
+        hide_progress=False,
+    ):
+        super().__init__(dir=dir, remote_url=remote_url, branch=branch, no_pull=no_pull, hide_progress=False)
 
+        self.fail_warned = fail_warned
         self.passed = []
         self.warned = []
         self.failed = []
-        self.modules_repo = ModulesRepo()
-        self.lint_tests = self._get_all_lint_tests()
-        # Get lists of modules install in directory
-        self.all_local_modules, self.all_nfcore_modules = self.get_installed_modules()
+        self.lint_tests = self.get_all_lint_tests(self.repo_type == "pipeline")
+
+        if self.repo_type == "pipeline":
+            modules_json = ModulesJson(self.dir)
+            modules_json.check_up_to_date()
+            all_pipeline_modules = modules_json.get_all_modules()
+            if self.modules_repo.fullname in all_pipeline_modules:
+                module_dir = Path(self.dir, "modules", self.modules_repo.fullname)
+                self.all_remote_modules = [
+                    NFCoreModule(m, self.modules_repo.fullname, module_dir / m, self.repo_type, Path(self.dir))
+                    for m in all_pipeline_modules[self.modules_repo.fullname]
+                ]
+                if not self.all_remote_modules:
+                    raise LookupError(f"No modules from {self.modules_repo.remote_url} installed in pipeline.")
+                local_module_dir = Path(self.dir, "modules", "local")
+                self.all_local_modules = [
+                    NFCoreModule(m, None, local_module_dir / m, self.repo_type, Path(self.dir), nf_core_module=False)
+                    for m in self.get_local_modules()
+                ]
+
+            else:
+                raise LookupError(f"No modules from {self.modules_repo.remote_url} installed in pipeline.")
+        else:
+            module_dir = Path(self.dir, "modules")
+            self.all_remote_modules = [
+                NFCoreModule(m, None, module_dir / m, self.repo_type, Path(self.dir))
+                for m in self.get_modules_clone_modules()
+            ]
+            self.all_local_modules = []
+            if not self.all_remote_modules:
+                raise LookupError("No modules in 'modules' directory")
 
         self.lint_config = None
         self.modules_json = None
 
-        # Add tests specific to nf-core/modules or pipelines
-        if self.repo_type == "modules":
-            self.lint_tests.append("module_tests")
-
-        if self.repo_type == "pipeline":
-            # Add as first test to load git_sha before module_changes
-            self.lint_tests.insert(0, "module_version")
-            # Only check if modules have been changed in pipelines
-            self.lint_tests.append("module_changes")
-
     @staticmethod
-    def _get_all_lint_tests():
-        return ["main_nf", "meta_yml", "module_todos", "module_deprecations"]
+    def get_all_lint_tests(is_pipeline):
+        if is_pipeline:
+            return [
+                "module_patch",
+                "module_version",
+                "main_nf",
+                "meta_yml",
+                "module_todos",
+                "module_deprecations",
+                "module_changes",
+            ]
+        else:
+            return ["main_nf", "meta_yml", "module_todos", "module_deprecations", "module_tests"]
 
-    def lint(self, module=None, key=(), all_modules=False, print_results=True, show_passed=False, local=False):
+    def lint(
+        self,
+        module=None,
+        key=(),
+        all_modules=False,
+        hide_progress=False,
+        print_results=True,
+        show_passed=False,
+        local=False,
+        fix_version=False,
+    ):
         """
         Lint all or one specific module
 
@@ -117,6 +156,8 @@ class ModuleLint(ModuleCommand):
         :param module:          A specific module to lint
         :param print_results:   Whether to print the linting results
         :param show_passed:     Whether passed tests should be shown as well
+        :param fix_version:     Update the module version if a newer version is available
+        :param hide_progress:   Don't show progress bars
 
         :returns:               A ModuleLint object containing information of
                                 the passed, warned and failed tests
@@ -136,7 +177,7 @@ class ModuleLint(ModuleCommand):
                     "name": "tool_name",
                     "message": "Tool name:",
                     "when": lambda x: x["all_modules"] == "Named module",
-                    "choices": [m.module_name for m in self.all_nfcore_modules],
+                    "choices": [m.module_name for m in self.all_remote_modules],
                 },
             ]
             answers = questionary.unsafe_prompt(questions, style=nf_core.utils.nfcore_question_style)
@@ -148,12 +189,12 @@ class ModuleLint(ModuleCommand):
             if all_modules:
                 raise ModuleLintException("You cannot specify a tool and request all tools to be linted.")
             local_modules = []
-            nfcore_modules = [m for m in self.all_nfcore_modules if m.module_name == module]
-            if len(nfcore_modules) == 0:
+            remote_modules = [m for m in self.all_remote_modules if m.module_name == module]
+            if len(remote_modules) == 0:
                 raise ModuleLintException(f"Could not find the specified module: '{module}'")
         else:
             local_modules = self.all_local_modules
-            nfcore_modules = self.all_nfcore_modules
+            remote_modules = self.all_remote_modules
 
         if self.repo_type == "modules":
             log.info(f"Linting modules repo: [magenta]'{self.dir}'")
@@ -173,11 +214,11 @@ class ModuleLint(ModuleCommand):
 
         # Lint local modules
         if local and len(local_modules) > 0:
-            self.lint_modules(local_modules, local=True)
+            self.lint_modules(local_modules, local=True, fix_version=fix_version)
 
         # Lint nf-core modules
-        if len(nfcore_modules) > 0:
-            self.lint_modules(nfcore_modules, local=False)
+        if len(remote_modules) > 0:
+            self.lint_modules(remote_modules, local=False, fix_version=fix_version)
 
         if print_results:
             self._print_results(show_passed=show_passed)
@@ -185,7 +226,8 @@ class ModuleLint(ModuleCommand):
 
     def set_up_pipeline_files(self):
         self.load_lint_config()
-        self.modules_json = self.load_modules_json()
+        self.modules_json = ModulesJson(self.dir)
+        self.modules_json.load()
 
         # Only continue if a lint config has been loaded
         if self.lint_config:
@@ -201,7 +243,7 @@ class ModuleLint(ModuleCommand):
         if len(bad_keys) > 0:
             raise AssertionError(
                 "Test name{} not recognised: '{}'".format(
-                    "s" if len(bad_keys) > 1 else "",
+                    _s(bad_keys),
                     "', '".join(bad_keys),
                 )
             )
@@ -209,90 +251,22 @@ class ModuleLint(ModuleCommand):
         # If -k supplied, only run these tests
         self.lint_tests = [k for k in self.lint_tests if k in key]
 
-    def get_installed_modules(self):
-        """
-        Makes lists of the local and and nf-core modules installed in this directory.
-
-        Returns:
-            local_modules, nfcore_modules ([NfCoreModule], [NfCoreModule]):
-                A tuple of two lists: One for local modules and one for nf-core modules.
-                In case the module contains several subtools, one path to each tool directory
-                is returned.
-
-        """
-        # Initialize lists
-        local_modules = []
-        nfcore_modules = []
-        local_modules_dir = None
-        nfcore_modules_dir = os.path.join(self.dir, "modules", "nf-core", "modules")
-
-        # Get local modules
-        if self.repo_type == "pipeline":
-            local_modules_dir = os.path.join(self.dir, "modules", "local")
-
-            # Filter local modules
-            if os.path.exists(local_modules_dir):
-                local_modules = sorted([x for x in local_modules if x.endswith(".nf")])
-
-        # nf-core/modules
-        if self.repo_type == "modules":
-            nfcore_modules_dir = os.path.join(self.dir, "modules")
-
-        # Get nf-core modules
-        if os.path.exists(nfcore_modules_dir):
-            for m in sorted(os.listdir(nfcore_modules_dir)):
-                if not os.path.isdir(os.path.join(nfcore_modules_dir, m)):
-                    raise ModuleLintException(
-                        f"File found in '{nfcore_modules_dir}': '{m}'! This directory should only contain module directories."
-                    )
-
-                module_dir = os.path.join(nfcore_modules_dir, m)
-                module_subdir = os.listdir(module_dir)
-                # Not a module, but contains sub-modules
-                if "main.nf" not in module_subdir:
-                    for path in module_subdir:
-                        module_subdir_path = os.path.join(nfcore_modules_dir, m, path)
-                        if os.path.isdir(module_subdir_path):
-                            if os.path.exists(os.path.join(module_subdir_path, "main.nf")):
-                                nfcore_modules.append(os.path.join(m, path))
-                else:
-                    nfcore_modules.append(m)
-
-        # Create NFCoreModule objects for the nf-core and local modules
-        nfcore_modules = [
-            NFCoreModule(os.path.join(nfcore_modules_dir, m), repo_type=self.repo_type, base_dir=self.dir)
-            for m in nfcore_modules
-        ]
-
-        local_modules = [
-            NFCoreModule(
-                os.path.join(local_modules_dir, m), repo_type=self.repo_type, base_dir=self.dir, nf_core_module=False
-            )
-            for m in local_modules
-        ]
-
-        # The local modules mustn't conform to the same file structure
-        # as the nf-core modules. We therefore only check the main script
-        # of the module
-        for mod in local_modules:
-            mod.main_nf = mod.module_dir
-            mod.module_name = os.path.basename(mod.module_dir)
-
-        return local_modules, nfcore_modules
-
-    def lint_modules(self, modules, local=False):
+    def lint_modules(self, modules, local=False, fix_version=False):
         """
         Lint a list of modules
 
         Args:
             modules ([NFCoreModule]): A list of module objects
             local (boolean): Whether the list consist of local or nf-core modules
+            fix_version (boolean): Fix the module version if a newer version is available
         """
         progress_bar = rich.progress.Progress(
             "[bold blue]{task.description}",
             rich.progress.BarColumn(bar_width=None),
             "[magenta]{task.completed} of {task.total}[reset] » [bold yellow]{task.fields[test_name]}",
             transient=True,
+            console=console,
+            disable=self.hide_progress,
         )
         with progress_bar:
             lint_progress = progress_bar.add_task(
@@ -303,9 +277,9 @@ class ModuleLint(ModuleCommand):
 
             for mod in modules:
                 progress_bar.update(lint_progress, advance=1, test_name=mod.module_name)
-                self.lint_module(mod, local=local)
+                self.lint_module(mod, progress_bar, local=local, fix_version=fix_version)
 
-    def lint_module(self, mod, local=False):
+    def lint_module(self, mod, progress_bar, local=False, fix_version=False):
         """
         Perform linting on one module
 
@@ -324,17 +298,29 @@ class ModuleLint(ModuleCommand):
 
         # Only check the main script in case of a local module
         if local:
-            self.main_nf(mod)
+            self.main_nf(mod, fix_version, progress_bar)
             self.passed += [LintResult(mod, *m) for m in mod.passed]
-            self.warned += [LintResult(mod, *m) for m in (mod.warned + mod.failed)]
+            warned = [LintResult(mod, *m) for m in (mod.warned + mod.failed)]
+            if not self.fail_warned:
+                self.warned += warned
+            else:
+                self.failed += warned
 
         # Otherwise run all the lint tests
         else:
             for test_name in self.lint_tests:
-                getattr(self, test_name)(mod)
+                if test_name == "main_nf":
+                    getattr(self, test_name)(mod, fix_version, progress_bar)
+                else:
+                    getattr(self, test_name)(mod)
 
             self.passed += [LintResult(mod, *m) for m in mod.passed]
-            self.warned += [LintResult(mod, *m) for m in mod.warned]
+            warned = [LintResult(mod, *m) for m in mod.warned]
+            if not self.fail_warned:
+                self.warned += warned
+            else:
+                self.failed += warned
+
             self.failed += [LintResult(mod, *m) for m in mod.failed]
 
     def _print_results(self, show_passed=False):
@@ -382,11 +368,6 @@ class ModuleLint(ModuleCommand):
                 )
             return table
 
-        def _s(some_list):
-            if len(some_list) > 1:
-                return "s"
-            return ""
-
         # Print blank line for spacing
         console.print("")
 
@@ -400,7 +381,7 @@ class ModuleLint(ModuleCommand):
             console.print(
                 rich.panel.Panel(
                     table,
-                    title=r"[bold][✔] {} Module Test{} Passed".format(len(self.passed), _s(self.passed)),
+                    title=rf"[bold][✔] {len(self.passed)} Module Test{_s(self.passed)} Passed",
                     title_align="left",
                     style="green",
                     padding=0,
@@ -417,7 +398,7 @@ class ModuleLint(ModuleCommand):
             console.print(
                 rich.panel.Panel(
                     table,
-                    title=r"[bold][!] {} Module Test Warning{}".format(len(self.warned), _s(self.warned)),
+                    title=rf"[bold][!] {len(self.warned)} Module Test Warning{_s(self.warned)}",
                     title_align="left",
                     style="yellow",
                     padding=0,
@@ -434,7 +415,7 @@ class ModuleLint(ModuleCommand):
             console.print(
                 rich.panel.Panel(
                     table,
-                    title=r"[bold][✗] {} Module Test{} Failed".format(len(self.failed), _s(self.failed)),
+                    title=rf"[bold][✗] {len(self.failed)} Module Test{_s(self.failed)} Failed",
                     title_align="left",
                     style="red",
                     padding=0,
@@ -442,18 +423,13 @@ class ModuleLint(ModuleCommand):
             )
 
     def print_summary(self):
-        def _s(some_list):
-            if len(some_list) > 1:
-                return "s"
-            return ""
-
-        # Summary table
+        """Print a summary table to the console."""
         table = Table(box=rich.box.ROUNDED)
-        table.add_column("[bold green]LINT RESULTS SUMMARY".format(len(self.passed)), no_wrap=True)
+        table.add_column("[bold green]LINT RESULTS SUMMARY", no_wrap=True)
         table.add_row(
-            r"[✔] {:>3} Test{} Passed".format(len(self.passed), _s(self.passed)),
+            rf"[✔] {len(self.passed):>3} Test{_s(self.passed)} Passed",
             style="green",
         )
-        table.add_row(r"[!] {:>3} Test Warning{}".format(len(self.warned), _s(self.warned)), style="yellow")
-        table.add_row(r"[✗] {:>3} Test{} Failed".format(len(self.failed), _s(self.failed)), style="red")
+        table.add_row(rf"[!] {len(self.warned):>3} Test Warning{_s(self.warned)}", style="yellow")
+        table.add_row(rf"[✗] {len(self.failed):>3} Test{_s(self.failed)} Failed", style="red")
         console.print(table)
