@@ -5,27 +5,28 @@ along with running the tests and creating md5 sums
 """
 
 from __future__ import print_function
-from rich.syntax import Syntax
 
 import errno
 import gzip
 import hashlib
+import io
 import logging
 import operator
 import os
-import questionary
 import re
-import rich
 import shlex
 import subprocess
 import tempfile
+
+import questionary
+import rich
 import yaml
+from rich.syntax import Syntax
 
 import nf_core.utils
 
 from .modules_repo import ModulesRepo
 
-
 log = logging.getLogger(__name__)
 
 
@@ -69,12 +70,11 @@ class ModulesTestYmlBuilder(object):
         # Get the tool name if not specified
         if self.module_name is None:
             modules_repo = ModulesRepo()
-            modules_repo.get_modules_file_tree()
             self.module_name = questionary.autocomplete(
                 "Tool name:",
-                choices=modules_repo.modules_avail_module_names,
+                choices=modules_repo.get_avail_modules(),
                 style=nf_core.utils.nfcore_question_style,
-            ).ask()
+            ).unsafe_ask()
         self.module_dir = os.path.join("modules", *self.module_name.split("/"))
         self.module_test_main = os.path.join("tests", "modules", *self.module_name.split("/"), "main.nf")
 
@@ -217,7 +217,7 @@ class ModulesTestYmlBuilder(object):
         """Generate md5 sum for file"""
         hash_md5 = hashlib.md5()
         with open(fname, "rb") as f:
-            for chunk in iter(lambda: f.read(4096), b""):
+            for chunk in iter(lambda: f.read(io.DEFAULT_BUFFER_SIZE), b""):
                 hash_md5.update(chunk)
         md5sum = hash_md5.hexdigest()
         return md5sum
@@ -225,7 +225,7 @@ class ModulesTestYmlBuilder(object):
     def create_test_file_dict(self, results_dir, is_repeat=False):
         """Walk through directory and collect md5 sums"""
         test_files = []
-        for root, dir, files in os.walk(results_dir, followlinks=True):
+        for root, _, files in os.walk(results_dir, followlinks=True):
             for filename in files:
                 # Check that the file is not versions.yml
                 if filename == "versions.yml":
@@ -263,7 +263,7 @@ class ModulesTestYmlBuilder(object):
                 results_dir, results_dir_repeat = self.run_tests_workflow(command)
             else:
                 results_dir = rich.prompt.Prompt.ask(
-                    f"[violet]Test output folder with results[/] (leave blank to run test)"
+                    "[violet]Test output folder with results[/] (leave blank to run test)"
                 )
                 if results_dir == "":
                     results_dir = None
@@ -325,7 +325,7 @@ class ModulesTestYmlBuilder(object):
         log.info(f"Running '{self.module_name}' test with command:\n[violet]{command}")
         try:
             nfconfig_raw = subprocess.check_output(shlex.split(command))
-            log.info(f"Repeating test ...")
+            log.info("Repeating test ...")
             nfconfig_raw = subprocess.check_output(shlex.split(command_repeat))
 
         except OSError as e:
@@ -363,4 +363,4 @@ class ModulesTestYmlBuilder(object):
             with open(self.test_yml_output_path, "w") as fh:
                 yaml.dump(self.tests, fh, Dumper=nf_core.utils.custom_yaml_dumper(), width=10000000)
         except FileNotFoundError as e:
-            raise UserWarning("Could not create test.yml file: '{}'".format(e))
+            raise UserWarning(f"Could not create test.yml file: '{e}'")
