@@ -2,19 +2,20 @@
 """Synchronise a pipeline TEMPLATE branch with the template.
 """
 
-import git
 import json
 import logging
 import os
+import shutil
+
+import git
 import requests
 import requests_cache
 import rich
-import shutil
+from git import GitCommandError, InvalidGitRepositoryError
 
 import nf_core
 import nf_core.create
 import nf_core.list
-import nf_core.sync
 import nf_core.utils
 
 log = logging.getLogger(__name__)
@@ -66,7 +67,7 @@ class PipelineSync(object):
         self.pipeline_dir = os.path.abspath(pipeline_dir)
         self.from_branch = from_branch
         self.original_branch = None
-        self.merge_branch = "nf-core-template-merge-{}".format(nf_core.__version__)
+        self.merge_branch = f"nf-core-template-merge-{nf_core.__version__}"
         self.made_changes = False
         self.make_pr = make_pr
         self.gh_pr_returned_data = {}
@@ -91,9 +92,9 @@ class PipelineSync(object):
         # Clear requests_cache so that we don't get stale API responses
         requests_cache.clear()
 
-        log.info("Pipeline directory: {}".format(self.pipeline_dir))
+        log.info(f"Pipeline directory: {self.pipeline_dir}")
         if self.from_branch:
-            log.info("Using branch '{}' to fetch workflow variables".format(self.from_branch))
+            log.info(f"Using branch '{self.from_branch}' to fetch workflow variables")
         if self.make_pr:
             log.info("Will attempt to automatically create a pull request")
 
@@ -130,9 +131,7 @@ class PipelineSync(object):
             log.info("No changes made to TEMPLATE - sync complete")
         elif not self.make_pr:
             log.info(
-                "Now try to merge the updates in to your pipeline:\n  cd {}\n  git merge TEMPLATE".format(
-                    self.pipeline_dir
-                )
+                f"Now try to merge the updates in to your pipeline:\n  cd {self.pipeline_dir}\n  git merge TEMPLATE"
             )
 
     def inspect_sync_dir(self):
@@ -142,12 +141,12 @@ class PipelineSync(object):
         # Check that the pipeline_dir is a git repo
         try:
             self.repo = git.Repo(self.pipeline_dir)
-        except git.exc.InvalidGitRepositoryError as e:
-            raise SyncException("'{}' does not appear to be a git repository".format(self.pipeline_dir))
+        except InvalidGitRepositoryError:
+            raise SyncException(f"'{self.pipeline_dir}' does not appear to be a git repository")
 
         # get current branch so we can switch back later
         self.original_branch = self.repo.active_branch.name
-        log.info("Original pipeline repository branch is '{}'".format(self.original_branch))
+        log.info(f"Original pipeline repository branch is '{self.original_branch}'")
 
         # Check to see if there are uncommitted changes on current branch
         if self.repo.is_dirty(untracked_files=True):
@@ -162,17 +161,17 @@ class PipelineSync(object):
         # Try to check out target branch (eg. `origin/dev`)
         try:
             if self.from_branch and self.repo.active_branch.name != self.from_branch:
-                log.info("Checking out workflow branch '{}'".format(self.from_branch))
+                log.info(f"Checking out workflow branch '{self.from_branch}'")
                 self.repo.git.checkout(self.from_branch)
-        except git.exc.GitCommandError:
-            raise SyncException("Branch `{}` not found!".format(self.from_branch))
+        except GitCommandError:
+            raise SyncException(f"Branch `{self.from_branch}` not found!")
 
         # If not specified, get the name of the active branch
         if not self.from_branch:
             try:
                 self.from_branch = self.repo.active_branch.name
-            except git.exc.GitCommandError as e:
-                log.error("Could not find active repo branch: ".format(e))
+            except GitCommandError as e:
+                log.error(f"Could not find active repo branch: {e}")
 
         # Fetch workflow variables
         log.debug("Fetching workflow config variables")
@@ -181,7 +180,7 @@ class PipelineSync(object):
         # Check that we have the required variables
         for rvar in self.required_config_vars:
             if rvar not in self.wf_config:
-                raise SyncException("Workflow config variable `{}` not found!".format(rvar))
+                raise SyncException(f"Workflow config variable `{rvar}` not found!")
 
     def checkout_template_branch(self):
         """
@@ -191,11 +190,11 @@ class PipelineSync(object):
         # Try to check out the `TEMPLATE` branch
         try:
             self.repo.git.checkout("origin/TEMPLATE", b="TEMPLATE")
-        except git.exc.GitCommandError:
+        except GitCommandError:
             # Try to check out an existing local branch called TEMPLATE
             try:
                 self.repo.git.checkout("TEMPLATE")
-            except git.exc.GitCommandError:
+            except GitCommandError:
                 raise SyncException("Could not check out branch 'origin/TEMPLATE' or 'TEMPLATE'")
 
     def delete_template_branch_files(self):
@@ -208,7 +207,7 @@ class PipelineSync(object):
             if the_file == ".git":
                 continue
             file_path = os.path.join(self.pipeline_dir, the_file)
-            log.debug("Deleting {}".format(file_path))
+            log.debug(f"Deleting {file_path}")
             try:
                 if os.path.isfile(file_path):
                     os.unlink(file_path)
@@ -234,6 +233,7 @@ class PipelineSync(object):
             force=True,
             outdir=self.pipeline_dir,
             author=self.wf_config["manifest.author"].strip('"').strip("'"),
+            plain=True,
         ).init_pipeline()
 
     def commit_template_changes(self):
@@ -245,11 +245,11 @@ class PipelineSync(object):
         # Commit changes
         try:
             self.repo.git.add(A=True)
-            self.repo.index.commit("Template update for nf-core/tools version {}".format(nf_core.__version__))
+            self.repo.index.commit(f"Template update for nf-core/tools version {nf_core.__version__}")
             self.made_changes = True
             log.info("Committed changes to 'TEMPLATE' branch")
         except Exception as e:
-            raise SyncException("Could not commit changes to TEMPLATE:\n{}".format(e))
+            raise SyncException(f"Could not commit changes to TEMPLATE:\n{e}")
         return True
 
     def push_template_branch(self):
@@ -257,11 +257,11 @@ class PipelineSync(object):
         and try to make a PR. If we don't have the auth token, try to figure out a URL
         for the PR and print this to the console.
         """
-        log.info("Pushing TEMPLATE branch to remote: '{}'".format(os.path.basename(self.pipeline_dir)))
+        log.info(f"Pushing TEMPLATE branch to remote: '{os.path.basename(self.pipeline_dir)}'")
         try:
             self.repo.git.push()
-        except git.exc.GitCommandError as e:
-            raise PullRequestException("Could not push TEMPLATE branch:\n  {}".format(e))
+        except GitCommandError as e:
+            raise PullRequestException(f"Could not push TEMPLATE branch:\n  {e}")
 
     def create_merge_base_branch(self):
         """Create a new branch from the updated TEMPLATE branch
@@ -279,16 +279,14 @@ class PipelineSync(object):
                 branch_no += 1
                 self.merge_branch = f"{original_merge_branch}-{branch_no}"
             log.info(
-                "Branch already existed: '{}', creating branch '{}' instead.".format(
-                    original_merge_branch, self.merge_branch
-                )
+                f"Branch already existed: '{original_merge_branch}', creating branch '{self.merge_branch}' instead."
             )
 
         # Create new branch and checkout
         log.info(f"Checking out merge base branch '{self.merge_branch}'")
         try:
             self.repo.create_head(self.merge_branch)
-        except git.exc.GitCommandError as e:
+        except GitCommandError as e:
             raise SyncException(f"Could not create new branch '{self.merge_branch}'\n{e}")
 
     def push_merge_branch(self):
@@ -297,7 +295,7 @@ class PipelineSync(object):
         try:
             origin = self.repo.remote()
             origin.push(self.merge_branch)
-        except git.exc.GitCommandError as e:
+        except GitCommandError as e:
             raise PullRequestException(f"Could not push branch '{self.merge_branch}':\n  {e}")
 
     def make_pull_request(self):
@@ -341,7 +339,7 @@ class PipelineSync(object):
             else:
                 self.gh_pr_returned_data = r.json()
                 self.pr_url = self.gh_pr_returned_data["html_url"]
-                log.debug(f"GitHub API PR worked, return code 201")
+                log.debug(f"GitHub API PR worked, return code {r.status_code}")
                 log.info(f"GitHub PR created: {self.gh_pr_returned_data['html_url']}")
 
     def close_open_template_merge_prs(self):
@@ -407,7 +405,7 @@ class PipelineSync(object):
 
         # PR update worked
         if pr_request.status_code == 200:
-            log.debug("GitHub API PR-update worked:\n{}".format(pr_request_pp))
+            log.debug(f"GitHub API PR-update worked:\n{pr_request_pp}")
             log.info(
                 f"Closed GitHub PR from '{pr['head']['ref']}' to '{pr['base']['ref']}': {pr_request_json['html_url']}"
             )
@@ -421,8 +419,8 @@ class PipelineSync(object):
         """
         Reset the target pipeline directory. Check out the original branch.
         """
-        log.info("Checking out original branch: '{}'".format(self.original_branch))
+        log.info(f"Checking out original branch: '{self.original_branch}'")
         try:
             self.repo.git.checkout(self.original_branch)
-        except git.exc.GitCommandError as e:
-            raise SyncException("Could not reset to original branch `{}`:\n{}".format(self.from_branch, e))
+        except GitCommandError as e:
+            raise SyncException(f"Could not reset to original branch `{self.from_branch}`:\n{e}")
