@@ -1,9 +1,3 @@
-import pytest
-import unittest
-import tempfile
-import os
-import shutil
-
 import nf_core.create
 import nf_core.lint
 
